#!/bin/bash

# A simple diagnostic script reporting outages in SPI communication
# unipi.technology @ 2021

# ================== DEFAULT VALUES ===================
# ==== CAN BE OVERRIDEN IN /etc/unipi/fw-diag-tool ====
CONFIG_FILE_PATH="/etc/unipi/fw-diag-tool.conf"
log_file="/opt/comm_outages.log"
REPORT_TO_SLACK=true # Comment to disable
REPORT_TO_DB=true  # Comment to disable
SUPRESS_SLACK_BOOTMSG=true
DISABLE_AUTOUPGRADE=false
#serial="/dev/ttyNS0" # Optional interconnection with DUT via an external RS485 line
SLACK_URL="https://hooks.slack.com/services/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" # Dead beef channel

#==== Internal variables ====
old_tick=0
counter=0
counter_reset=60
has_section_2=false
has_section_3=false
section_1_reported=false
section_2_reported=false
section_3_reported=false
no_space_reported=false
machine_name=$(hostname)

version_str="0.18" # Placeholder to be replaced in CI script by the right value

function remote_report_msg {
    #Remote log server
    if [[ $REPORT_TO_DB = true ]]; then
      msg=$(echo "${1}" | tr -d '\n')
      logger -P 514 -n 81.19.0.22 -d -t BRAIN -p 2 --rfc5424=notq "${msg}"
    fi

    #Slack API
    if [[ $REPORT_TO_SLACK = true ]]; then
        if ! curl -X POST -H 'Content-type: application/json' --data "{\"text\": \" ${machine_name}(v.${version_str}): ${1}\n\"}" "$SLACK_URL" >/dev/null 2>&1; then
            local_report_msg "Could not send: $1"
        fi
    fi
}

function local_report_msg {
    # check if there is enough space for logging (in kB)
    if [ "$(df "$log_file" | awk 'NR==2{print $4}')" -ge 25000 ]; then
        echo "$(date "+%Y-%m-%d %H:%M:%S") $1" >> $log_file
        if [ $no_space_reported == true ]; then
            remote_report_msg "Free space magically appeared for $log_file"
            no_space_reported=false
        fi
    else
        # no, report it
        if [ $no_space_reported == false ]; then
            msg="No space left for $log_file\n$(df -h)"
            echo "$msg"
            remote_report_msg "$msg"
            no_space_reported=true
        fi
    fi
}

function report_both {
    local_report_msg "$1"
    remote_report_msg "$1"
}

function do_autoupgrade { # TODO - move to a separate dynamically-created executable file

    mount -o remount, rw /
    apt update
    available_version=$(apt-cache show fw-diag-tool --no-all-versions | grep -o "Version:[[:space:]][[:digit:]]*.[[:digit:]]*" | cut -d " " -f 2)

    if [ ! "$version_str" == "$available_version" ]; then
        remote_report_msg "Auto upgrade from $version_str to $available_version"
        apt install -y fw-diag-tool
        sleep 5
        exit 0
    fi
}

# do some preliminary test of the HW
if [ -f /opt/unipi/tools/fwspi ]; then
    # is there a brain?
    HAS_BRAIN=$(/opt/unipi/tools/fwspi -i 0 | grep -c "B-1000")
    if [ "$HAS_BRAIN" == "0" ]; then
        # nope, we shouldn't run at all
        NO_BRAIN="This thing does not have a brain, exiting"
        remote_report_msg "$NO_BRAIN"
        echo "$NO_BRAIN"
        exit
    fi

    # change the COUNTER_REG based on the presence of B-485 firmware
    HAS_B485=$(/opt/unipi/tools/fwspi -i 0 | grep -c "B-485")
    if [ "$HAS_B485" == "1" ]; then
        COUNTER_REG=1043
    else
        COUNTER_REG=1033
    fi
else
    echo "No unipi-modbus-tools, exiting"
    exit
fi

echo -n "Waiting 10s for settling down..... "
sleep 10
echo "done"

# Load configuration file
if [[ -f "${CONFIG_FILE_PATH}" ]]; then
    source "${CONFIG_FILE_PATH}"
fi

# override the machine name, if provided from the config file
# otherwise use the hostname
if [[ -n "${FORCE_MACHINE_NAME}" ]]; then
    machine_name="${FORCE_MACHINE_NAME}"
fi

# log and send message during the boot time
if [ "$(cut -d"." -f 1 /proc/uptime)" -le 60 ]; then
    msg="BOOT (fw-diag-tool v.${version_str})"
    local_report_msg "$msg"
    # if the boot message shouldn't be suppressed, send it via Slack as well
    if [ "${SUPRESS_SLACK_BOOTMSG}" == false ]; then
        remote_report_msg "$msg"
    fi
fi

# Verify serial line
if [[ ! -c "${serial}" ]]; then
    serial=""
fi

# Verify 2 and 3 groups presence
mbpoll 127.0.0.1 -a 2 -0 -1 -r 1000 -q >/dev/null 2>&1 && has_section_2=true
mbpoll 127.0.0.1 -a 3 -0 -1 -r 1000 -q >/dev/null 2>&1 && has_section_3=true

while true; do
    # Check group 1 - BRAIN is always present
    # check the firmware version
    if ! mbpoll 127.0.0.1 -a 1 -0 -1 -r 1000 -q >/dev/null 2>/dev/null; then
        msg="Section 1: SPI outage; last tick: ${tick}"

        # if we have a serial connection, check the state over the serial as well
        if [ -n "${serial}" ] && [ "$HAS_B485" == "1" ]; then
            msg="$msg\n$(mbpoll /dev/ttyNS0 -a 15 -P none -b 9600 -o 0.1 -0 -r 1033 -c 13 -1 -q 2>&1)"
        fi

        # report the status based on the counter
        if [ "$counter" == "0" ]; then
            # each 60 cycles (seconds) of the downtime, report to both
            report_both "$msg"
            counter=$counter_reset
        else
            # every other cycle report to local only
            local_report_msg "$msg"
        fi
        # increase (well, decrease) the counter of the downtime
        counter=$((counter-1))

        # mark that we have sent OUTAGE message, for future sending OK message
        section_1_reported=true
    else
        if [ $section_1_reported == true ]; then
            report_both "Section 1: SPI is back"
            section_1_reported=false
        fi

        # Catch watchdog reset flag
        if mbpoll 127.0.0.1 -a 1 -01 -r 1000 -q -t0 2>/dev/null | grep -q -e '\s1$'; then
            # reset watchdog bit
            mbpoll 127.0.0.1 -a 1 -01 -r 1000 -q -t0 1 2>/dev/null >/dev/null
            report_both "Section 1: watchdog reset; last brain tick: ${tick}"
        fi

        # Catch firmware main loop counter reset
        tick=$(mbpoll "127.0.0.1" -a 1 -0 -t 4:int -r $COUNTER_REG -c 1 -1 -q | grep -o '[[:digit:]]*$')
        tick=$(( tick / 4  ))
        if [ $tick -lt $old_tick ]; then
            report_both "Section 1: tick reset; old tick: $old_tick; new tick: $tick"
        fi
        old_tick=$tick

        # reset counter, so next error will send message immediatelly
        counter=0
    fi

    # Check group 2 - capture the change of the availability
    if [ $has_section_2 == true ] && ! mbpoll 127.0.0.1 -a 2 -0 -1 -r 1000 -q >/dev/null 2>/dev/null ; then
        if [ $section_2_reported == false ]; then
            report_both "Section 2: SPI outage"
            section_2_reported=true
        fi
    elif [ $section_2_reported == true ]; then
        report_both "Section 2: SPI is back"
        section_2_reported=false
    fi

    # Check group 3 - capture the change of the availability
    if [ $has_section_3 == true ] && ! mbpoll 127.0.0.1 -a 3 -0 -1 -r 1000 -q >/dev/null 2>/dev/null ; then
        if [ $section_3_reported == false ]; then
            report_both "Section 3: SPI outage"
            section_3_reported=true
        fi
    elif [ $section_3_reported == true ]; then
        report_both "Section 3: SPI is back"
        section_3_reported=false
    fi

    sleep 1
done
